//
// bidwatcher
// copyright (c) 1999 2000
// Trent McNair (trent@rmci.net)
// Tom McNair  (tmcnair@cyberhighway.net)
// Wayne Schlitt (wayne@midwestcs.com)
// 
// use of this code is restricted to the terms
// of the GPL, which should have been included in this
// distribution. If not, see www.gnu.org/copyleft/gpl.html.
// Here is the short version:
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#include <signal.h>
#include "bidwatcher.h"

bool cancelPressed;
double avg_load_time = INITIAL_LOAD_TIME;

char cookie[512];

void CloseSocket(int sockIT)
{
   close(sockIT);
}
int set_nonblock(int fd)
{
    int flags = fcntl(fd, F_GETFL, 0);
    if (flags == -1)
        return -1;
    return fcntl(fd, F_SETFL, flags | O_NONBLOCK);
}
//////////////////////////////////////////////////////////////////////////////
//  Fetches an auction from ebay and stores the content (html tags are
//  removed) into <Buff>
//
//  Return codes are as follows:
//				  1 - success
//				  2 - connect failure 
//				  4 - timeout
//				  10 - cancelled by user
//////////////////////////////////////////////////////////////////////////////
int raw_FetchHtml(char *WebPage, char *Buff, const char * theAddress, int sizeOfBuff, int timeOut)
{
   int    buffLength,recerr;
   int    port=80;
   int    socketID;
   char   lineBuff[1024];
   struct    sockaddr_in soc_in;
   int	  err;
   fd_set fds;
   struct timeval tm;

   QTime time_since_start; 
   time_since_start.start();

   strcpy(Buff, "");
   memset( (char*)&soc_in, 0, sizeof(soc_in) );
   soc_in.sin_family = AF_INET;
   memcpy ((char *)&soc_in.sin_addr,theAddress,4);
   soc_in.sin_port = htons(port);
//   socketID = socket(2, SOCK_STREAM, 0);
   socketID = socket(AF_INET, SOCK_STREAM, 0);
   if (socketID < 0) return 2;

   set_nonblock(socketID);   
   err = ::connect(socketID, (struct sockaddr *)&soc_in, sizeof (soc_in));

   if ( err < 0 )
   {
       if( errno == EINPROGRESS )
       {
	   FD_ZERO(&fds);
	   FD_SET(socketID,&fds);
	   tm.tv_sec = timeOut / (4 * 1000) + 1;
	   tm.tv_usec = ((timeOut / 4) % 1000) * 1000;
	   err = select(socketID+1,NULL,&fds,NULL,&tm);

	   if( err == 0 )
	   {
#ifdef DEBUG_NETWORK
	       fprintf( stderr, "connection timed out\n" );
#endif
	       CloseSocket(socketID);
	       return 4;
	   }
       }
       else
       {
#ifdef DEBUG_NETWORK
	   fprintf( stderr,
		    "connection failed immediately:  err=%d  errno=%d (%s)\n",
		    err, errno, strerror( errno ) );
#endif
	   CloseSocket(socketID);
	   return 2;
       }
   }

   // did we connect, or not? 
   err = 0;

   {
       socklen_t arglen = sizeof(int);

       /* this is slightly grotty to avoid problems with SysV, where    */
       /* apparently getsockopt() will fail with an async error, rather */
       /* that return it like it's supposed to.                         */

       if (getsockopt(socketID,SOL_SOCKET,SO_ERROR,&err,&arglen) < 0)
	   err = errno;
   }
   
   if( err != 0 )
   {
#ifdef DEBUG_NETWORK
       fprintf( stderr,
		"connection failed:  err=%d  errno=%d (%s)\n",
		err, errno, strerror( errno ) );
#endif
       CloseSocket(socketID);
       return 2;
   }

   // send our request for the web page.
   sprintf( lineBuff, "GET %s HTTP/1.0\r\nHost: %s\r\n\r\n", WebPage, &theAddress[4]);
   if( send(socketID, lineBuff, strlen(lineBuff), 0 ) < 0 )
   {
#ifdef DEBUG_NETWORK
       fprintf( stderr,
		"could not send URL:  errno=%d (%s)\n",
		errno, strerror( errno ) );
#endif
       CloseSocket(socketID);
       return 2;
   }

   buffLength = 0;

   int counter = 0;
   bool goodcall = TRUE;
   QTime time_since_pkt;
   time_since_pkt.start();
   while ( goodcall )
   {
      if ( cancelPressed ) return 10;
      if ( counter == 5 )
      {
	  counter = 0;
	  qApp->processEvents();
	  if ( time_since_start.elapsed() > timeOut )
	  {
	      CloseSocket(socketID);
	      return 4;
	  }

	  if ( time_since_pkt.elapsed() > timeOut/2 )
	  {
	      CloseSocket(socketID);
	      return 4;
	  }
      }
      recerr = recv(socketID, lineBuff, 256, 0);
      if ( recerr < 0 )
      {
         if ( errno != EWOULDBLOCK )
         {
	     goodcall = FALSE;
	     break;
         }
         goodcall = TRUE;
	 qApp->processEvents();
	 usleep(10000);
	 counter++;
      }
      else if ( recerr == 0 )
	  break;
      else
      {
	  if ( (buffLength + recerr) < sizeOfBuff )
	  {
	      time_since_pkt.restart();
	      counter = 0;
	      buffLength = buffLength + recerr;
	      lineBuff[256] = '\0';
	      strncat(Buff, lineBuff, recerr);
	      goodcall = TRUE;
	  }
	  else break;
      }
   }
   CloseSocket(socketID);
   // there is a slight chance that the web page was _exactly the
   // same size as the buffer, so we will  fix that:
   Buff[sizeOfBuff - 1] = '\0';
   if ( !goodcall ) return 2; 
   return 1; 
} 

int FetchHtml(char *WebPage, char *Buff, const char * theAddress,
	      int sizeOfBuff, int timeOut)
{
    int	err;

    struct timeval tm_start, tm_end;
    double	t1;

#ifdef DEBUG_NETWORK
    char	*page_start;
#define MAX_CNTS    10
    int		j;
    static int		t_cnts[ MAX_CNTS ];
    static double	t_total = 0;
    static int		t_num = 0;
#endif

    Buff[0] = '\0';

    // adjust the timeout based on how well the network is working

    if( avg_load_time > NORMAL_LOAD_TIME )
    {
#ifdef DEBUG_NETWORK
	fprintf( stderr, "poor network response:  "
		 "Adjusting timeout from %d to %d\n",
		 timeOut, timeOut + (int)avg_load_time - NORMAL_LOAD_TIME );
#endif
	timeOut += (int)avg_load_time - NORMAL_LOAD_TIME;
    }

    for( int i = 0; i < 5; i++ )
    {
	gettimeofday( &tm_start, NULL );

	err = raw_FetchHtml(WebPage, Buff, theAddress, sizeOfBuff, timeOut);
  if (strstr(Buff,"Invalid Item")!=NULL) return 11;

  if (strlen(Buff) < 800) {
  char *loc;
   char authID[76];
   char authPASS[76];

  if (strstr(Buff,"HTTP/1.0 30")!=NULL || strstr(Buff,"HTTP/1.1 30")!=NULL) {
    loc=strstr(Buff,"Location:");
    if (loc == NULL) {
    loc=strstr(Buff,"location:");
     
      loc += 29;  // strlen("location: ");
      loc[strcspn(loc,"\r\n ")]='\0';

    if (strstr(loc,"AdultLoginShow")) {
#ifdef DEBUG_NETWORK
      printf("Getting adult cookie\n");
#endif
      getAdultCookie(authID,authPASS);
      if (strlen(cookie)<5) printf("%s\n","Could not get adult cookie.");
    }
    }
    else
    {
      loc += 29;  // strlen("Location: ");
    if (loc[0]!='/' ) loc +=1;
    loc[strcspn(loc,"\r\n ")]='\0';

    if (strstr(loc,"AdultLoginShow")) {
#ifdef DEBUG_NETWORK
      printf("Getting adult cookie\n");
#endif
      getAdultCookie(authID,authPASS);
      if (strlen(cookie)<5) printf("%s\n","Could not get adult cookie.");
    }

    }
      strcpy(WebPage, loc);
      strcpy(loc, "");
      err = raw_FetchHtml(WebPage, Buff, theAddress, sizeOfBuff, timeOut);
}
}
	gettimeofday( &tm_end, NULL );
	t1 = (tm_end.tv_sec + 0.000001 * tm_end.tv_usec)
	    - (tm_start.tv_sec + 0.000001 * tm_start.tv_usec);

	// keep a running avergae.  Respond quicker to spikes up than
	// drops in the delay.
	if( avg_load_time < t1 )
	    avg_load_time = avg_load_time * .75 + t1 * 1000 * .25;
	else
	    avg_load_time = avg_load_time * .90 + t1 * 1000 * .10;

#ifdef DEBUG_NETWORK
	if( (int)t1 >= MAX_CNTS  )
	    t_cnts[ MAX_CNTS-1 ]++;
	else
	    t_cnts[ (int)t1 ]++;

	t_total += t1;
	t_num++;
	if( err != 1 || i != 0 )
	{
	    if( strncmp( WebPage, "/ws/eBayISAPI.dll?",
			 sizeof( "/ws/eBayISAPI.dll?" ) - 1 ) == 0 )
		page_start = WebPage + sizeof( "/ws/eBayISAPI.dll?" ) - 1;
	    else
		page_start = WebPage;

	    fprintf( stderr, "%d:   Err=%d  web page \"%s\"  "
		     "  time: %.2f/%.2f\n",
		     i, err, page_start, t1, timeOut/1000. );

	    fprintf( stderr, "     len=%d  Buff=%.80s\n",
		     strlen( Buff ), Buff );

	    if( err == 1 )
	    {
		fprintf( stderr, "avg_load_time:  %.2f   t_avg: %.2f  %d",
			 avg_load_time, 1000 * t_total/t_num, t_cnts[ 0 ] );
		for( j = 1; j < MAX_CNTS; j++ )
		    fprintf( stderr, "/%d", t_cnts[ j ] );
		fprintf( stderr, "\n" );
		fprintf( stderr, "-----------------------------------------------------------------\n\n" );
	    }
	    else
		fprintf( stderr, "\n" );
	}
#endif

	switch( err )
	{
	default:
	case 1:
	case 10:
	    return err;
	    break;

	case 2:
	    sleep( 2 );
	    break;

	case 4:
	    break;
	}

	timeOut *= 2;
    }

#ifdef DEBUG_NETWORK
    if( err )
	fprintf( stderr, "!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!--\n\n" );
#endif
    return err;
}

char WebPage2[80];

void getAdultCookie(char * username, char * password) {



   char authID[76];
   char authPASS[76];
   bool dataID;
   char fileName[200];
    
   ifstream inFile;
   char * homedir = getenv("HOME");
   strcpy(fileName, homedir);
   strcat(fileName, "/.bidwatcher/bw.cfg");
 
   inFile.open(fileName, ios::in);

   if ( inFile )
   {
      inFile >> dataID;
      while ( inFile )
      {
         switch (dataID)
         {
   
            case 1:
               inFile.ignore(200,'\n');
                inFile >> dataID;
               inFile.ignore();
               inFile.getline(authID, 76, '\n');
               break;
                inFile >> dataID;
               inFile.ignore();
               inFile.getline(authPASS, 76, '\n');
               break;

        default:
                inFile.close();
                break;
          }
                inFile.close();
         }
                inFile.close();
        }

sprintf( WebPage2, "/aw-cgi/eBayISAPI.dll"
                        "?MfcISAPICommand=AdultLogin&userid=%s&"
                        "password=%s", username, password);

    char Buff[HUGEBUFF];
    strcpy(Buff, "");
    int err,sizeOfBuff;
    sizeOfBuff = strlen(Buff);

    err = raw_FetchHtml(WebPage2, Buff, theAddress, sizeOfBuff, TIMEOUT);
    char *adultcookie=strstr(Buff,"Set-Cookie");
    if (adultcookie==NULL) {
      printf("adult cookie not found\n");
      printf(Buff);
      return;
    }
    int len=strcspn(adultcookie,";");
    adultcookie[len]='\0';
    printf("Adult cookie: %s\n",adultcookie+4); // strip "Set-"
    strcpy(cookie,adultcookie+4);
}

